@@ -229,7 +229,8 @@ class Wrapper:
             if function_obj.ret not in ('none', None):
                 substdict['setreturn'] = 'ret = '
             handler = argtypes.matcher.get(function_obj.ret)
-            handler.write_return(function_obj.ret, info)
+            handler.write_return(function_obj.ret,
+                                 function_obj.caller_owns_return, info)
 
         if function_obj.deprecated != None:
             deprecated = self.deprecated_tmpl % {
@@ -376,7 +377,8 @@ class Wrapper:
                     funcname = getterprefix + fname
                     info = argtypes.WrapperInfo()
                     handler = argtypes.matcher.get(ftype)
-                    handler.write_return(ftype, info)
+                    # for attributes, we don't own the "return value"
+                    handler.write_return(ftype, 0, info)
                     self.fp.write(self.getter_tmpl %
                                   { 'funcname': funcname,
                                     'varlist': info.varlist,
@@ -451,7 +453,7 @@ class GObjectWrapper(Wrapper):
         '        PyErr_SetString(PyExc_RuntimeError, "could not create %(typename)s object");\n' \
         '        return -1;\n' \
         '    }\n' \
-        '%(gtkobjectsink)s' \
+        '%(aftercreate)s' \
         '    pygobject_register_wrapper((PyObject *)self);\n' \
         '    return 0;\n' \
         '}\n\n'
@@ -483,12 +485,10 @@ class GObjectWrapper(Wrapper):
 
     def get_initial_constructor_substdict(self):
         substdict = Wrapper.get_initial_constructor_substdict(self)
-        if argtypes.matcher.object_is_a(self.objinfo.c_name, 'GtkObject'):
-            substdict['gtkobjectsink'] = \
-                '    g_object_ref(self->obj);\n' \
-                '    gtk_object_sink(GTK_OBJECT(self->obj));\n'
+        if argtypes.matcher.object_is_a(self.objinfo.c_name, 'GtkWindow'):
+            substdict['aftercreate'] = "    g_object_ref(self->obj); /* we don't own the first reference of windows */\n"
         else:
-            substdict['gtkobjectsink'] = ''
+            substdict['aftercreate'] = ''
         return substdict
 
     def get_initial_method_substdict(self, method):
